unit UMainForm;

{$MODE Delphi}

interface

uses
  SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, USourceCode, UCompiler, UProducerAsm8051,
  UProducerC, ComCtrls, ExtCtrls, SynEdit, SynHighlighterPas, UProducerTreeView,
  UProducerPascal, Contnrs;

const
  ExampleFileName = '../../Example.pas';

type
  { TMainForm }

  TMainForm = class(TForm)
    ButtonCompile: TButton;
    ComboBox1: TComboBox;
    ListBox1: TListBox;
    Panel1: TPanel;
    SynEdit1: TSynEdit;
    SynPasSyn1: TSynPasSyn;
    procedure ComboBox1Change(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure ButtonCompileClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ListBox1Click(Sender: TObject);
    procedure LoadErrorMessages;
  private
  public
    ProducerControl: TControl;
    Compiler: TCompiler;
    SourceTree: TProducerTreeView;
  end;

var
  MainForm: TMainForm;

implementation

{$R *.lfm}

{ TMainForm }

procedure TMainForm.ButtonCompileClick(Sender: TObject);
var
  I: Integer;
begin
  MainForm.ListBox1.Clear;
  Compiler.SourceCode.Assign(SynEdit1.Lines);

  ProducerControl.Free;
  if ComboBox1.ItemIndex = 0 then begin
    Compiler.Producer.Free;
    Compiler.Producer := TProducerPascal.Create;
    Compiler.Producer.ProgramCode := Compiler.ProgramCode;
    Compiler.Compile;
    ProducerControl := TMemo.Create(Self);
    ProducerControl.Parent := Panel1;
    ProducerControl.Align := alClient;
    ProducerControl.Font.Name := 'Courier New';
    TMemo(ProducerControl).ScrollBars := ssAutoBoth;
    with TProducerPascal(Compiler.Producer) do
      TMemo(ProducerControl).Lines.Assign(TextSource);
  end else
  if ComboBox1.ItemIndex = 1 then begin
    Compiler.Producer.Free;
    Compiler.Producer := TProducerC.Create;
    Compiler.Producer.ProgramCode := Compiler.ProgramCode;
    Compiler.Compile;
    ProducerControl := TMemo.Create(Self);
    ProducerControl.Parent := Panel1;
    ProducerControl.Align := alClient;
    ProducerControl.Font.Name := 'Courier New';
    TMemo(ProducerControl).ScrollBars := ssAutoBoth;
    with TProducerC(Compiler.Producer) do
      TMemo(ProducerControl).Lines.Assign(TextSource);
  end else
  if ComboBox1.ItemIndex = 2 then begin
    Compiler.Producer.Free;
    Compiler.Producer := TProducerAsm8051.Create;
    Compiler.Producer.ProgramCode := Compiler.ProgramCode;
    Compiler.Compile;
    ProducerControl := TMemo.Create(Self);
    ProducerControl.Parent := Panel1;
    ProducerControl.Align := alClient;
    ProducerControl.Font.Name := 'Courier New';
    TMemo(ProducerControl).ScrollBars := ssAutoBoth;
    with TProducerAsm8051(Compiler.Producer) do
    for I := 0 to AssemblyCode.Count - 1 do
      TMemo(ProducerControl).Lines.Add(TAssemblerLine(AssemblyCode[I]).AsString);
  end;
  if ComboBox1.ItemIndex = 3 then begin
    Compiler.Producer.Free;
    Compiler.Producer := TProducerTreeView.Create;
    Compiler.Producer.ProgramCode := Compiler.ProgramCode;
    ProducerControl := TTreeView.Create(Self);
    ProducerControl.Parent := Panel1;
    ProducerControl.Align := alClient;
    TProducerTreeView(Compiler.Producer).TreeView := TTreeView(ProducerControl);
    Compiler.Compile;
  end;
  LoadErrorMessages;
end;

procedure TMainForm.ListBox1Click(Sender: TObject);
begin
  if ListBox1.ItemIndex <> - 1 then begin
    SynEdit1.CaretXY := TErrorMessage(Compiler.ErrorMessages[ListBox1.ItemIndex]).Position;
    SynEdit1.SetFocus;
  end;
end;

procedure TMainForm.LoadErrorMessages;
var
  I: Integer;
begin
  ListBox1.Clear;
  for I := 0 to Compiler.ErrorMessages.Count - 1 do
  with TErrorMessage(Compiler.ErrorMessages[I]) do
    MainForm.ListBox1.Items.Add('(' + IntToStr(Position.X) +
      ',' + IntToStr(Position.Y) + ') ' + Text);
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  SynEdit1.Lines.SaveToFile(ExampleFileName);
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  Compiler := TCompiler.Create;
  SourceTree := TProducerTreeView.Create;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  Compiler.Free;
  SourceTree.Free;
end;

procedure TMainForm.FormShow(Sender: TObject);
begin
  WindowState := wsMaximized;
  SynEdit1.Lines.LoadFromFile(ExampleFileName);
  ButtonCompileClick(Self);
end;

procedure TMainForm.ComboBox1Change(Sender: TObject);
begin
  ButtonCompileClick(Self);
end;

end.
